==> tao-0.2.2.crate: tao-0.2.2/examples/control_flow.rs <==
// Copyright 2019-2021 Tauri Programme within The Commons Conservancy
// SPDX-License-Identifier: Apache-2.0

use std::{thread, time};

use simple_logger::SimpleLogger;
use tao::{
  event::{Event, KeyboardInput, WindowEvent},
  event_loop::{ControlFlow, EventLoop},
  window::WindowBuilder,
};

#[derive(Debug, Clone, Copy, PartialEq, Eq)]
enum Mode {
  Wait,
  WaitUntil,
  Poll,
}

const WAIT_TIME: time::Duration = time::Duration::from_millis(100);
const POLL_SLEEP_TIME: time::Duration = time::Duration::from_millis(100);

#[allow(clippy::single_match)]
fn main() {
  SimpleLogger::new().init().unwrap();

  println!("Press '1' to switch to Wait mode.");
  println!("Press '2' to switch to WaitUntil mode.");
  println!("Press '3' to switch to Poll mode.");
  println!("Press 'R' to toggle request_redraw() calls.");
  println!("Press 'Esc' to close the window.");

  let event_loop = EventLoop::new();
  let window = WindowBuilder::new()
    .with_title("Press 1, 2, 3 to change control flow mode. Press R to toggle redraw requests.")
    .build(&event_loop)
    .unwrap();

  let mut mode = Mode::Wait;
  let mut request_redraw = false;
  let mut wait_cancelled = false;
  let mut close_requested = false;

  event_loop.run(move |event, _, control_flow| {
    use tao::event::{ElementState, StartCause, VirtualKeyCode};
    println!("{:?}", event);
    match event {
      Event::NewEvents(start_cause) => {
        wait_cancelled = match start_cause {
          StartCause::WaitCancelled { .. } => mode == Mode::WaitUntil,
          _ => false,
        }
      }
      Event::WindowEvent { event, .. } => match event {
        WindowEvent::CloseRequested => {
          close_requested = true;
        }
        WindowEvent::KeyboardInput {
          input:
            KeyboardInput {
              virtual_keycode: Some(virtual_code),
              state: ElementState::Pressed,
              ..
            },
          ..
        } => match virtual_code {
          VirtualKeyCode::Key1 => {
            mode = Mode::Wait;
            println!("\nmode: {:?}\n", mode);
          }
          VirtualKeyCode::Key2 => {
            mode = Mode::WaitUntil;
            println!("\nmode: {:?}\n", mode);
          }
          VirtualKeyCode::Key3 => {
            mode = Mode::Poll;
            println!("\nmode: {:?}\n", mode);
          }
          VirtualKeyCode::R => {
            request_redraw = !request_redraw;
            println!("\nrequest_redraw: {}\n", request_redraw);
          }
          VirtualKeyCode::Escape => {
            close_requested = true;
          }
          _ => (),
        },
        _ => (),
      },
      Event::MainEventsCleared => {
        if request_redraw && !wait_cancelled && !close_requested {
          window.request_redraw();
        }
        if close_requested {
          *control_flow = ControlFlow::Exit;
        }
      }
      Event::RedrawRequested(_window_id) => {}
      Event::RedrawEventsCleared => {
        *control_flow = match mode {
          Mode::Wait => ControlFlow::Wait,
          Mode::WaitUntil => {
            if wait_cancelled {
              *control_flow
            } else {
              ControlFlow::WaitUntil(time::Instant::now() + WAIT_TIME)
            }
          }
          Mode::Poll => {
            thread::sleep(POLL_SLEEP_TIME);
            ControlFlow::Poll
          }
        };
      }
      _ => (),
    }
  });
}

==> tao-0.4.0.crate: tao-0.4.0/examples/control_flow.rs <==
// Copyright 2019-2021 Tauri Programme within The Commons Conservancy
// SPDX-License-Identifier: Apache-2.0

use std::{thread, time};

use simple_logger::SimpleLogger;
use tao::{
  event::{ElementState, Event, KeyEvent, WindowEvent},
  event_loop::{ControlFlow, EventLoop},
  keyboard::Key,
  window::WindowBuilder,
};

#[derive(Debug, Clone, Copy, PartialEq, Eq)]
enum Mode {
  Wait,
  WaitUntil,
  Poll,
}

const WAIT_TIME: time::Duration = time::Duration::from_millis(100);
const POLL_SLEEP_TIME: time::Duration = time::Duration::from_millis(100);

#[allow(clippy::single_match)]
fn main() {
  SimpleLogger::new().init().unwrap();

  println!("Press '1' to switch to Wait mode.");
  println!("Press '2' to switch to WaitUntil mode.");
  println!("Press '3' to switch to Poll mode.");
  println!("Press 'R' to toggle request_redraw() calls.");
  println!("Press 'Esc' to close the window.");

  let event_loop = EventLoop::new();
  let window = WindowBuilder::new()
    .with_title("Press 1, 2, 3 to change control flow mode. Press R to toggle redraw requests.")
    .build(&event_loop)
    .unwrap();

  let mut mode = Mode::Wait;
  let mut request_redraw = false;
  let mut wait_cancelled = false;
  let mut close_requested = false;

  event_loop.run(move |event, _, control_flow| {
    use tao::event::StartCause;
    println!("{:?}", event);
    match event {
      Event::NewEvents(start_cause) => {
        wait_cancelled = match start_cause {
          StartCause::WaitCancelled { .. } => mode == Mode::WaitUntil,
          _ => false,
        }
      }
      Event::WindowEvent { event, .. } => match event {
        WindowEvent::CloseRequested => {
          close_requested = true;
        }
        WindowEvent::KeyboardInput {
          event:
            KeyEvent {
              logical_key,
              state: ElementState::Pressed,
              ..
            },
          ..
        } => {
          // WARNING: Consider using `key_without_modifers()` if available on your platform.
          // See the `key_binding` example
          if Key::Character("1") == logical_key {
            mode = Mode::Wait;
            println!("\nmode: {:?}\n", mode);
          }
          if Key::Character("2") == logical_key {
            mode = Mode::WaitUntil;
            println!("\nmode: {:?}\n", mode);
          }
          if Key::Character("3") == logical_key {
            mode = Mode::Poll;
            println!("\nmode: {:?}\n", mode);
          }
          if Key::Character("r") == logical_key {
            request_redraw = !request_redraw;
            println!("\nrequest_redraw: {}\n", request_redraw);
          }
          if Key::Escape == logical_key {
            close_requested = true;
          }
        }
        _ => {}
      },
      Event::MainEventsCleared => {
        if request_redraw && !wait_cancelled && !close_requested {
          window.request_redraw();
        }
        if close_requested {
          *control_flow = ControlFlow::Exit;
        }
      }
      Event::RedrawRequested(_window_id) => {}
      Event::RedrawEventsCleared => {
        *control_flow = match mode {
          Mode::Wait => ControlFlow::Wait,
          Mode::WaitUntil => {
            if wait_cancelled {
              *control_flow
            } else {
              ControlFlow::WaitUntil(time::Instant::now() + WAIT_TIME)
            }
          }
          Mode::Poll => {
            thread::sleep(POLL_SLEEP_TIME);
            ControlFlow::Poll
          }
        };
      }
      _ => (),
    }
  });
}
